@@ -187,8 +187,11 @@ help_cb (BonoboUIComponent   *uic,
 		&error);
 
 	if (error) { 
+		GtkWidget *parent =
+			gtk_widget_get_parent (GTK_WIDGET (sapplet->applet));
+
 		GtkWidget *dialog = 
-			gtk_message_dialog_new (GTK_WINDOW (sapplet->applet),
+			gtk_message_dialog_new (GTK_WINDOW (parent),
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_CLOSE,
@@ -908,7 +911,7 @@ accessx_applet_add_stock_icons (AccessxStatusApplet *sapplet, GtkWidget *widget)
 		do {
 			char *filename, *tmp;
 			GtkIconSource *source = gtk_icon_source_new ();
-			tmp = g_strdup_printf (PIXMAPDIR"accessx-status-applet/%s", stock_icons[i].name);
+			tmp = g_strdup_printf (GNOME_PIXMAPSDIR"accessx-status-applet/%s", stock_icons[i].name);
 			filename = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
 					  tmp, 
 					  FALSE, NULL);
@@ -1306,7 +1309,7 @@ accessx_status_applet_fill (PanelApplet *applet)
 				   G_CALLBACK (key_press_cb), sapplet);				   
 
 	panel_applet_setup_menu_from_file (sapplet->applet,
-                                           NULL,
+                                           DATADIR,
 				           "GNOME_AccessxApplet.xml",
                                            NULL,
 				           accessx_status_applet_menu_verbs,
