@@ -18,7 +18,7 @@
  *
  * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
  *
-$Id: properties.c,v 1.40 2004/10/26 14:17:56 davyd Exp $
+$Id: properties.c,v 1.42 2005/01/11 06:17:09 ryanl Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -128,12 +128,8 @@ show_bat_toggled (GtkToggleButton *button, gpointer data)
   }
 
   battstat->showbattery = toggled;
-  if (battstat->horizont) 
-    battstat->showbattery ?
-	gtk_widget_show (battstat->pixmapwid):gtk_widget_hide (battstat->pixmapwid);
-  else
-    battstat->showbattery ?
-	gtk_widget_show (battstat->frameybattery):gtk_widget_hide (battstat->frameybattery);
+  reconfigure_layout( battstat );
+
   panel_applet_gconf_set_bool   (applet, "show_battery", 
   				 battstat->showbattery, NULL);
   				 
@@ -154,7 +150,7 @@ show_status_toggled (GtkToggleButton *button, gpointer data)
   }
   
   battstat->showstatus = toggled;
-	change_orient(applet, battstat->orienttype, battstat);
+  reconfigure_layout( battstat );
   
   panel_applet_gconf_set_bool   (applet, "show_status", 
   				 battstat->showstatus, NULL);
@@ -177,8 +173,8 @@ show_text_toggled (GtkToggleButton *button, gpointer data)
 	  battstat->showtext = APPLET_SHOW_TIME;
   else
 	  battstat->showtext = APPLET_SHOW_NONE;
-  
-  change_orient(applet, battstat->orienttype, battstat);
+ 
+  reconfigure_layout( battstat ); 
 
   gtk_widget_set_sensitive (GTK_WIDGET (battstat->radio_text_1),
 		  battstat->showtext);
@@ -297,7 +293,6 @@ prop_cb (BonoboUIComponent *uic,
   GtkWidget *layout_table;
   GtkWidget *preview_hbox;
   GtkWidget *widget;
-  guint      percentage;
   GConfClient *client;
   gboolean   inhibit_command_line;
   AtkObject *atk_widget;
@@ -305,21 +300,6 @@ prop_cb (BonoboUIComponent *uic,
   client = gconf_client_get_default ();
   inhibit_command_line = gconf_client_get_bool (client, "/desktop/gnome/lockdown/inhibit_command_line", NULL);
 
-  apm_readinfo (PANEL_APPLET (battstat->applet), battstat);
-
-#ifdef __FreeBSD__
-  percentage = apminfo.ai_batt_life;
-  if(percentage == 255) percentage = 0;
-#elif defined(__NetBSD__) || defined(__OpenBSD__)
-  percentage = apminfo.battery_life;
-  if(percentage == 255) percentage = 0;
-#elif __linux__
-  percentage = apminfo.battery_percentage;
-  if(percentage < 0) percentage = 0;
-#else
-  percentage = 100;
-#endif
-
   if (DEBUG) g_print("prop_cb()\n");
 
    if (battstat->prop_win) { 
@@ -329,7 +309,7 @@ prop_cb (BonoboUIComponent *uic,
      return;
    } 
 
-  glade_xml = glade_xml_new (GLADE_DIR "/battstat_applet.glade", 
+  glade_xml = glade_xml_new (GNOME_GLADEDIR "/battstat_applet.glade", 
                              "battstat_properties", NULL);
   
   battstat->prop_win = GTK_DIALOG (glade_xml_get_widget (glade_xml, 
