@@ -6,8 +6,10 @@
 #include <dirent.h>
 #include <string.h>
 #include <time.h>
-#include <gnome.h>
+#include <glib.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <gnome.h>
 #include <panel-applet.h>
 
 #include "global.h"
@@ -102,8 +104,7 @@ load_graph_update (LoadGraph *g)
 	g->get_data (g->draw_height, g->data [0], g);
 
     for (i=0; i < g->draw_width-1; i++)
-		for (j=0; j < g->n; j++)
-		    g->data [i+1][j] = g->odata [i][j];
+	    memcpy(g->data [i+1], g->odata [i], g->n * sizeof g->odata [i][0]);
 
     load_graph_draw (g);
     return TRUE;
@@ -138,6 +139,11 @@ load_graph_unalloc (LoadGraph *g)
 		g->pixmap = NULL;
     }
 
+    if (g->gc) {
+		g_object_unref (g->gc);
+		g->gc = NULL;
+    }
+
     g->allocated = FALSE;
 }
 
@@ -222,6 +228,11 @@ load_graph_destroy (GtkWidget *widget, gpointer data_ptr)
     LoadGraph *g = (LoadGraph *) data_ptr;
 	
     load_graph_stop (g);
+    if (g->tooltips) {
+    		g_object_unref (g->tooltips);
+		g->tooltips = NULL;
+    }
+
     gtk_widget_destroy(widget);
     object_list = g_list_remove (object_list, g);
     return;
@@ -319,7 +330,9 @@ load_graph_new (PanelApplet *applet, guint n, gchar *label,
         gtk_widget_set_size_request (g->main_widget, g->size, -1);
 
     g->tooltips = gtk_tooltips_new();
-   	
+    g_object_ref (g->tooltips);
+    gtk_object_sink (GTK_OBJECT (g->tooltips));
+
     g->disp = gtk_drawing_area_new ();
     gtk_widget_set_events (g->disp, GDK_EXPOSURE_MASK | GDK_ENTER_NOTIFY_MASK 
     				    | GDK_LEAVE_NOTIFY_MASK);
